export async function saveFileToOpfs(fsFilePath, opfsFilePath) {

    const opfsRoot = await navigator.storage.getDirectory();
    
    if (window.Module.FS.analyzePath(fsFilePath).exists) {
        
        const waitFlush = new Promise((done, _) => {
            setTimeout(done, 10);
        });

        await waitFlush;
        
        const data = window.Module.FS.readFile(fsFilePath);
        
        try{
            await opfsRoot.removeEntry(opfsFilePath);
        }catch (e) {
            console.warn(e);
        }
        
        let fileHandle = await opfsRoot.getFileHandle(opfsFilePath, {create: true});
        const accessHandle = await fileHandle.createWritable();
        await accessHandle.write(data);
        await accessHandle.close();
        window.Module.FS.unlink(fsFilePath);

        return true;
    }

    console.error(`Saving error ${fsFilePath} not found`);
    
    
    return false;
}

export async function restoreFileFromOpfs(fsFilePath, opfsFilePath) {

    const opfsRoot = await navigator.storage.getDirectory();
    let data = undefined;
    let exist = false;
    try {
        data = await opfsRoot.getFileHandle(opfsFilePath, {create: false});
        exist = true;
    } catch (error) {
        console.error(error);
    }

    if (exist) {

        const file = await data.getFile();
        if (file) {
            let array = (await file).arrayBuffer();
            let byteArray = new Int8Array(await array);
            window.Module.FS.writeFile(fsFilePath,byteArray);
            return true;
        }
    }
    return false;
}
export async function removeFileFromOpfs(file){
    const opfsRoot = await navigator.storage.getDirectory();
    await opfsRoot.removeEntry(file);
}
export async function downloadFileFromOpfs(file) {
    
    const opfsRoot = await navigator.storage.getDirectory();
    let fileHandle = await opfsRoot.getFileHandle(file, {create: false});
    const data = await fileHandle.getFile();
    
    const url = URL.createObjectURL(data);
    triggerFileDownload(file, url);
    URL.revokeObjectURL(url);


}
function triggerFileDownload(fileName, url) {
    const anchorElement = document.createElement('a');
    anchorElement.href = url;

    if (fileName) {
        anchorElement.download = fileName;
    }

    anchorElement.click();
    anchorElement.remove();
}

export async function saveDataToOpfs(data, opfsFilePath) {
    const opfsRoot = await navigator.storage.getDirectory();
    
    if (data) {
        let fileHandle = await opfsRoot.getFileHandle(opfsFilePath, {create: true});
        const accessHandle = await fileHandle.createWritable();

        await accessHandle.truncate(0);
        await accessHandle.write(await data.arrayBuffer());
        await accessHandle.close();

        window.sqlitedb = {
            init: false
        };
        return true;
    }

    return false;
}


